#Copyright 2010-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# This file is licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License. A copy of the
# License is located at
#
# http://aws.amazon.com/apache2.0/
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
# OF ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
#
# ABOUT THIS PYTHON SAMPLE: This sample is part of the AWS General Reference 
# Signing AWS API Requests top available at
# https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html
#

# AWS Version 4 signing example

# EC2 API (DescribeRegions)

# See: http://docs.aws.amazon.com/general/latest/gr/sigv4_signing.html
# This version makes a GET request and passes the signature
# in the Authorization header.
import sys, os, base64, datetime, hashlib, hmac,time
import requests # pip install requests

# ************* REQUEST VALUES *************
method = 'GET'
service = 'api'
region = 'rc'
host = ".".join([service, region, "antavo.com"])
# e.g., host = 'api.rc.antavo.com'
content_type = 'application/x-www-form-urlencoded'
request_parameters = ''
# Create canonical URI--the part of the URI from domain to query 
# string (use '/' if no path)
canonical_uri = '' 
# e.g., canonical_uri = '/rewards' 

# Step 3: Create the canonical query string. In this example (a GET request),
# request parameters are in the query string. Query string values must
# be URL-encoded (space=%20, comma=%2C). The parameters must be sorted by name.
# For this example, the query string is pre-formatted in the request_parameters variable.
canonical_querystring = ''
#e.g.,  canonical_querystring = 'max_price=125&min_price=50'
#Alphabetically sort the canonical_querystring
unsorted_cq = canonical_querystring.split("&")
unsorted_cq.sort()
canonical_querystring = "&".join(unsorted_cq)

if canonical_querystring:
    endpoint = "https://" + host + canonical_uri + "?" + canonical_querystring

else:
    endpoint = "https://" + host + canonical_uri #e.g., endpoint = 'https://api.rc.antavo.com/'
#e.g., endpoint = 'https://api.rc.antavo.com/'

# Create a date for headers and the credential string
t = datetime.datetime.utcnow()
date = t.strftime('%Y%m%dT%H%M%SZ')
date_stamp = t.strftime('%Y%m%d')  # Date w/o time, used in credential scope
# date = '20210331T182310Z'
# date_stamp = '2021031'
# Read AWS access key from env. variables or configuration file. Best practice is NOT
# to embed credentials in code.
access_key = '***********'
secret_key = '***********'

# Key derivation functions. See:
# http://docs.aws.amazon.com/general/latest/gr/signature-v4-examples.html#signature-v4-examples-python
def sign(key, msg):
    return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

def getSignatureKey(key, dateStamp, regionName, serviceName):
    kDate = sign(('ANTAVO' + key).encode('utf-8'), dateStamp)
    kRegion = sign(kDate, regionName)
    kService = sign(kRegion, serviceName)
    kSigning = sign(kService, 'antavo_request')
    return kSigning


# ************* TASK 1: CREATE A CANONICAL REQUEST *************
# http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html


canonical_headers = 'date:' + date + '\n' + 'host:' + host + '\n'
signed_headers = 'date;host'

# Step 6: Create payload hash (hash of the request body content). For GET
# requests, the payload is an empty string ("").
payload_hash = hashlib.sha256(('').encode('utf-8')).hexdigest()

# Step 7: Combine elements to create canonical request
canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash


# ************* TASK 2: CREATE THE STRING TO SIGN*************
# Match the algorithm to the hashing algorithm you use, either SHA-1 or
# SHA-256 (recommended)
algorithm = 'ANTAVO-HMAC-SHA256'
credential_scope = date_stamp + '/' + region + '/' + service + '/' + 'antavo_request'
string_to_sign = algorithm + '\n' +  date + '\n' +  credential_scope + '\n' +  hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()

# ************* TASK 3: CALCULATE THE SIGNATURE *************
# Create the signing key using the function defined above.
signing_key = getSignatureKey(secret_key, date_stamp, region, service)

# Sign the string_to_sign using the signing_key
signature = hmac.new(signing_key, (string_to_sign).encode('utf-8'), hashlib.sha256).hexdigest()

print(signature)

# ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
# The signing information can be either in a query string value or in 
# a header named Authorization. This code shows how to use a header.
# Create authorization header and add to request headers
authorization_header = algorithm + ' ' + 'Credential=' + access_key + '/' + credential_scope + ', ' +  'SignedHeaders=' + signed_headers + ', ' + 'Signature=' + signature

# The request can include any headers, but MUST include "host", "x-amz-date", 
# and (for this scenario) "Authorization". "host" and "x-amz-date" must
# be included in the canonical_headers and signed_headers, as noted
# earlier. Order here is not significant.
# Python note: The 'host' header is added automatically by the Python 'requests' library.
headers = {
    'Content-Type': content_type,
    'date': date,
    'Authorization': authorization_header
}


# ************* SEND THE REQUEST *************
request_url = endpoint #+ '?' + canonical_querystring

print('\nBEGIN REQUEST++++++++++++++++++++++++++++++++++++')
print('Request URL = ' + request_url)

start = time.time()
r = requests.get(request_url, headers=headers)
roundtrip = time.time() - start

print('\nRESPONSE++++++++++++++++++++++++++++++++++++')
print('Response code: %d\n' % r.status_code)
print(r.text)
 
print("Time taken - {}".format(r.elapsed.total_seconds()))
print("Time with content - {}".format(roundtrip))
